      SUBROUTINE USR_RATES(IJK, RATES)

      USE compar
      USE constant
      USE energy
      USE fldvar
      USE fun_avg
      USE functions
      USE funits
      USE geometry
      USE indices
      USE parallel
      USE param
      USE param1
      USE physprop
      USE run
      USE rxns
      USE sendrecv
      USE toleranc
      USE usr

      IMPLICIT NONE

      INTEGER, INTENT(IN) :: IJK

      DOUBLE PRECISION, DIMENSION(NO_OF_RXNS), INTENT(OUT) :: RATES

! Reaction specific variables:
!`````````````````````````````````````````````````````````````````````//
! Gas phase concentrations (mol/m^3)
      DOUBLE PRECISION  c_H2, c_CH4, c_H2O, c_CO, c_CO2
! Partial pressures (KPa)
      DOUBLE PRECISION  p_H2, p_H2O
! User defined
      DOUBLE PRECISION r_f, r_b, k_eq

! Prevent reactions when mass fraction falls below c_Limiter.
      DOUBLE PRECISION, parameter :: c_Limiter = 1.0d-7

!`````````````````````````````````````````````````````````````````````//
      INCLUDE 'species.inc'
!`````````````````````````````````````````````````````````````````````//


! Initialize partial pressures and concentrations.

      c_H2    = ZERO;   p_H2   = ZERO;
      c_H2O   = ZERO;   p_H2O  = ZERO;
      c_CH4   = ZERO;
      c_CO    = ZERO;
      c_CO2   = ZERO;	

! Calculate the partial pressure and molar concentration of H2
      IF(X_g(IJK,H2) > c_Limiter) then
         c_H2 = RO_g(IJK) * X_g(IJK,H2) / Mw_g(H2)
      ENDIF

! Calculate the molar concentration of H2O
      IF(X_g(IJK,H2O) > c_Limiter) then
            c_H2O = RO_g(IJK)*X_g(IJK,H2O)/Mw_g(H2O)
      ENDIF

! Calculate the molar concentration of CH4
      IF(X_g(IJK,CH4) > c_Limiter) then
            c_CH4 = RO_g(IJK)*X_g(IJK,CH4)/Mw_g(CH4)
      ENDIF    
        
! Calculate the molar concentration of CO
      IF(X_g(IJK,CO) > c_Limiter) then
            c_CO = RO_g(IJK)*X_g(IJK,CO)/Mw_g(CO)
      ENDIF 

! Calculate the molar concentration of CO2
      IF(X_g(IJK,CO2) > c_Limiter) then
            c_CO2 = RO_g(IJK)*X_g(IJK,CO2)/Mw_g(CO2)
      ENDIF 

!**********************************************************************!
!                         Homogeneous Reactions                        !
!**********************************************************************!
! CO + H2O <--> CO2 + H2  (kmole/m^3.s)
!---------------------------------------------------------------------//
      !RATES(Rg_CO) = 0.029 * exp( 3.40d7/(8314*T_g(IJK) ) )
      r_f = 2.78d3 * c_H2O * c_CO * exp( -1.26d7/(GAS_CONST*T_g(IJK) ) )
      r_b = 9.59d4 * c_CO2 * c_H2 * exp( -4.66d7/(GAS_CONST*T_g(IJK) ) )
      k_eq= 0.029 * exp( 3.40d7/(GAS_CONST*T_g(IJK) ) )
      RATES(Rg_CO) = r_f - r_b/k_eq

! CH4 + H2O --> CO + 3*H2  (kmole/m^3.s)
!---------------------------------------------------------------------//
      RATES(Rg_CH4)= 3d8 * c_CH4 * c_H2O * EXP(-1.26d8/GAS_CONST/T_g(IJK))
      

! Save reaction rates for visualization

      !IF(nrr>=Rg1)  ReactionRates(IJK,Rg1)  = RATES(Rg1)
      ReactionRates(IJK,1) = RATES(Rg_CO)
      ReactionRates(IJK,2) = RATES(Rg_CH4)
      RETURN

      END SUBROUTINE USR_RATES
